#ifndef KADU_H
#define KADU_H

#include <qmainwindow.h>
#include <qstring.h>
#include <qvaluelist.h>

#include "gadu.h"
#include "status.h"
#include "usergroup.h"
#include "userlistelement.h"

class QFile;
class QListBoxItem;
class QMenuBar;
class KaduTabBar;
class UserBox;
class KaduTextBrowser;
class QPushButton;

/**
	Gwne okno Kadu
**/
class Kadu : public QMainWindow
{
	Q_OBJECT

	private:
		static bool Closing;
		friend class KaduSlots;
		KaduTextBrowser* InfoPanel;
		QMenuBar* MenuBar;
		QPopupMenu* MainMenu;
		KaduTabBar* GroupBar;
		UserBox* Userbox;

		QPopupMenu* statusMenu;
		QPushButton* statusButton;

		UserStatus status;

		bool ShowMainWindowOnStart;
		bool DoBlink;
		bool BlinkOn;
		bool Docked;
		bool dontHideOnClose;

		void createMenu();
		void createToolBar();
		void createStatusPopupMenu();

		void showStatusOnMenu(int);

	public slots:
		void show();
		void mouseButtonClicked(int, QListBoxItem *);
		void updateInformationPanel(UserListElement);
		void sendMessage(UserListElement elem);
		void configure();

	private slots:
		void openChat();
		void chatMsgReceived(Protocol *protocol, UserListElements senders, const QString &msg, time_t time);

		void wentOnline(const QString &);
		void wentBusy(const QString &);
		void wentInvisible(const QString &);
		void wentOffline(const QString &);
		void connected();
		void connecting();
		void disconnected();
		void imageReceivedAndSaved(UinType sender, uint32_t size, uint32_t crc32, const QString &path);
		void systemMessageReceived(const QString &msg);
		void userStatusChanged(UserListElement user, QString protocolName, const UserStatus &oldstatus, bool massively, bool last);
		void deleteOldConfigFiles();

		void userDataChangedSlot(UserListElement elem, QString name, QVariant oldValue,
							QVariant currentValue, bool massively, bool last);
		void protocolUserDataChangedSlot(QString protocolName, UserListElement elem,
							QString name, QVariant oldValue, QVariant currentValue,
							bool massively, bool last);
		void currentChanged(UserListElement);

	protected:
		void keyPressEvent(QKeyEvent *e);
		virtual void resizeEvent(QResizeEvent *);
//		virtual void moveEvent(QMoveEvent *);

	public:
		Kadu(QWidget* parent=0, const char *name=0);
		~Kadu();
		bool userInActiveGroup(UinType uin);
		void removeUsers(UserListElements);

		/**
			Zwraca wskaznik do belki menu glownego okna.
		**/
		QMenuBar* menuBar() const;

		/**
			Zwraca wskaznik do glownego menu programu.
		**/
		QPopupMenu* mainMenu() const;

		/**
			Zwraca wskaznik do zakladek z nazwami grup.
		**/
		KaduTabBar* groupBar() const;

		/**
			Zwraca wskaznik do userbox-a w glownym oknie.
		**/
		UserBox* userbox() const;

		/**
			Zwraca true jeli kadu jest zadokowane.
		**/
		bool docked() const;

		/**
		**/
		void startupProcedure();

		/**
		**/
		int personalInfoMenuId;//potrzebne dla moduu account_management

		/**
			w zwizku z tym, e opis skadni dla parsera jest uywany w kilku miejscach
			dodane zostao to pole, eby nie trzeba byo zmienia tekstu w kilku miejscach
		**/
		static const QString SyntaxText;

		static bool closing() { return Closing; }
		static void setClosing() { Closing = true; }
	public slots:
		void slotHandleState(int command);
		void changeAppearance();
		void blink();
		void showdesc(bool show = true);
		void statusMenuAboutToHide(void);
		virtual bool close(bool quit = false);
		void quitApplication();

		/**
			Potrzebne dla moduu dokujcego eby
			gwne okno nie migao przy starcie...
		**/
		void setShowMainWindowOnStart(bool show);

		/**
			Modu dokowania powinien to ustawic, aby kadu
			wiedzialo, ze jest zadokowane.
		**/
		void setDocked(bool docked, bool dontHideOnClose);

		void about();
		void addUserAction();
		void blockUser();
		void deleteHistory();
		void deleteUsers();
		void help();
		void hideKadu();
		void ignoreUser();
		void importExportUserlist();
		void lookupInDirectory();
		void manageIgnored();
		void notifyUser();
		void offlineToUser();
		void personalInfo();
		void quit();
		void searchInDirectory();
		void showUserInfo();
		void viewHistory();
		void popupMenu();

		// odczytuje z obrazka tekst i zapisuje go w drugim parametrze
		void readTokenValue(QPixmap, QString &);

		void setMainWindowIcon(const QPixmap &);

	signals:
		void disconnectingNetwork();
		void disconnectedNetwork();
		void keyPressed(QKeyEvent* e);
		void statusPixmapChanged(const QPixmap &icon, const QString &icon_name);

		/**
			Uytkownik chce zmieni status. Jeli jest to status
			bezopisowy, to zmienna sigDesc bdzie rwna QString::null.
			Jeli zostanie ustawiona zmienna stop, to status nie
			bdzie zmieniony.
		**/
		void changingStatus(UserStatus &, bool &stop);

		/**
			wywoana zostaa funkcja show() na gwnym oknie
		**/
		void shown();

		/**
			uywany przez modu hints do zdobycia pozycji traya od moduu docking
			TODO: trzeba wymyli jaki elegancki sposb na komunikacj pomidzy moduami, ktre nie zale od siebie
		**/
		void searchingForTrayPosition(QPoint &point);

		void settingMainIconBlocked(bool &);
};

class KaduSlots : public QObject
{
	Q_OBJECT

	public:
		KaduSlots(QObject *parent=0, const char *name=0);
	public slots:
		void onCreateConfigDialog();
		void onDestroyConfigDialog();
		void chooseColor(const char *name, const QColor& color);
		void chooseFont(const char *name, const QFont& font);
		void updatePreview();
		void updateStatus(bool);
};

extern Kadu* kadu;
extern QPopupMenu* dockMenu;
extern int lockFileHandle;
extern QFile *lockFile;

#endif
